--
-- Baler
--
-- @author  Stefan Geiger
-- @date  10/09/08
--
-- Copyright (C) GIANTS Software GmbH, Confidential, All Rights Reserved.
--
-- cottonBaler mod script by fcelsa 13/01/2018
-- a rewrite of baler to make cotton bales from a combine
--
-- v 29

source(g_currentModDirectory .. "scripts/cottonBalerSetIsUnloadingBaleEvent.lua")
source(g_currentModDirectory .. "scripts/cottonBalerCreateBaleEvent.lua")
cottonBaler= {}

cottonBaler.UNLOADING_CLOSED = 1
cottonBaler.UNLOADING_OPENING = 2
cottonBaler.UNLOADING_OPEN = 3
cottonBaler.UNLOADING_CLOSING = 4

--function cottonBaler.initSpecialization()
    --WorkArea.registerAreaType("baler")
--end

function cottonBaler.prerequisitesPresent(specializations)
    -- fcelsa: cottonBaler require to be a combine harvester 
    return SpecializationUtil.hasSpecialization(Combine, specializations)
end

function cottonBaler:preLoad(savegame)
    --self.loadWorkAreaFromXML = Utils.overwrittenFunction(self.loadWorkAreaFromXML, cottonBaler.loadWorkAreaFromXML)
    self.loadSpeedRotatingPartFromXML = Utils.overwrittenFunction(self.loadSpeedRotatingPartFromXML, cottonBaler.loadSpeedRotatingPartFromXML)
end

function cottonBaler:load(savegame)

    self.modDir = g_currentModDirectory

    -- fcelsa see if need this
    Utils.checkDeprecatedXMLElements(self.xmlFile, self.configFileName, "vehicle.fillScale#value", "vehicle.baler#fillScale")
    Utils.checkDeprecatedXMLElements(self.xmlFile, self.configFileName, "vehicle.baleAnimation", "vehicle.baler.baleAnimation")
    Utils.checkDeprecatedXMLElements(self.xmlFile, self.configFileName, "vehicle.baleAnimation", "vehicle.baler.baleAnimation")
    Utils.checkDeprecatedXMLElements(self.xmlFile, self.configFileName, "vehicle.balerUVScrollParts.balerUVScrollPart", "vehicle.baler.uvScrollParts.uvScrollPart")
    Utils.checkDeprecatedXMLElements(self.xmlFile, self.configFileName, "vehicle.knotingAnimation", "vehicle.baler.knotingAnimation")
    Utils.checkDeprecatedXMLElements(self.xmlFile, self.configFileName, "vehicle.balingAnimation", "vehicle.baler.balingAnimation")
    Utils.checkDeprecatedXMLElements(self.xmlFile, self.configFileName, "vehicle.baleUnloading#allowed", "vehicle.baler.baleUnloading#allowed")
    Utils.checkDeprecatedXMLElements(self.xmlFile, self.configFileName, "vehicle.baleUnloading#time", "vehicle.baler.baleUnloading#time")
    Utils.checkDeprecatedXMLElements(self.xmlFile, self.configFileName, "vehicle.baleUnloading#foldThreshold", "vehicle.baler.baleUnloading#foldThreshold")

    self.doCheckSpeedLimit = Utils.overwrittenFunction(self.doCheckSpeedLimit, cottonBaler.doCheckSpeedLimit)
    self.getIsSpeedRotatingPartActive = Utils.overwrittenFunction(self.getIsSpeedRotatingPartActive, cottonBaler.getIsSpeedRotatingPartActive)
    -- fcelsa no pickup spec allowed here
    self.allowPickingUp = cottonBaler.allowPickingUp --Utils.overwrittenFunction(self.allowPickingUp, cottonBaler.allowPickingUp)
    --self.getIsTurnedOnAllowed = Utils.overwrittenFunction(self.getIsTurnedOnAllowed, cottonBaler.getIsTurnedOnAllowed)
    --self.getIsFoldAllowed = Utils.overwrittenFunction(self.getIsFoldAllowed, cottonBaler.getIsFoldAllowed)
    self.setUnitFillLevel = Utils.appendedFunction(self.setUnitFillLevel, cottonBaler.setUnitFillLevel)
    self.isUnloadingAllowed = cottonBaler.isUnloadingAllowed
    self.getTimeFromLevel = cottonBaler.getTimeFromLevel
    self.moveBales = SpecializationUtil.callSpecializationsFunction("moveBales")
    self.moveBale = SpecializationUtil.callSpecializationsFunction("moveBale")
    --self.allowFillType = cottonBaler.allowFillType
    self.setIsUnloadingBale = cottonBaler.setIsUnloadingBale
    self.dropBale = cottonBaler.dropBale
    self.createBale = cottonBaler.createBale
    self.setBaleTime = cottonBaler.setBaleTime
    self.processBalerAreas = cottonBaler.processBalerAreas

    self.cottonBaler = {}
    self.cottonBaler.fillScale = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.fillScale#value"), 1)
    self.cottonBaler.fillUnitIndex = Utils.getNoNil(getXMLInt(self.xmlFile, "vehicle.baler#fillUnitIndex"), 1)
    self.cottonBaler.unloadInfoIndex = Utils.getNoNil(getXMLInt(self.xmlFile, "vehicle.baler#unloadInfoIndex"), 1)
    self.cottonBaler.loadInfoIndex = Utils.getNoNil(getXMLInt(self.xmlFile, "vehicle.baler#loadInfoIndex"), 1)
    self.cottonBaler.dischargeInfoIndex = Utils.getNoNil(getXMLInt(self.xmlFile, "vehicle.baler#dischargeInfoIndex"), 1)

    local firstBaleMarker = getXMLFloat(self.xmlFile, "vehicle.baler.baleAnimation#firstBaleMarker")
    if firstBaleMarker ~= nil then
        local baleAnimCurve = AnimCurve:new(linearInterpolatorN)
        local keyI = 0
        while true do
            local key = string.format("vehicle.baler.baleAnimation.key(%d)", keyI)
            local t = getXMLFloat(self.xmlFile, key.."#time")
            local x,y,z = Utils.getVectorFromString(getXMLString(self.xmlFile, key.."#pos"))
            if x == nil or y == nil or z == nil then
                break
            end
            local rx, ry, rz = Utils.getVectorFromString(getXMLString(self.xmlFile, key.."#rot"))
            rx = math.rad(Utils.getNoNil(rx, 0))
            ry = math.rad(Utils.getNoNil(ry, 0))
            rz = math.rad(Utils.getNoNil(rz, 0))
            baleAnimCurve:addKeyframe({ v={x, y, z, rx, ry, rz}, time = t})
            keyI = keyI +1
        end
        if keyI > 0 then
            self.cottonBaler.baleAnimCurve = baleAnimCurve
            self.cottonBaler.firstBaleMarker = firstBaleMarker
        end
    end
    self.cottonBaler.baleAnimRoot, self.cottonBaler.baleAnimRootComponent = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.baler.baleAnimation#node"))
    if self.cottonBaler.baleAnimRoot == nil then
        self.cottonBaler.baleAnimRoot = self.components[1].node
        self.cottonBaler.baleAnimRootComponent = self.components[1].node
    end
    -- there is no standard bale animation, load the unload animation (for round baler)
    if self.cottonBaler.firstBaleMarker == nil then
        local unloadAnimationName = getXMLString(self.xmlFile, "vehicle.baler.baleAnimation#unloadAnimationName")
        local closeAnimationName = getXMLString(self.xmlFile, "vehicle.baler.baleAnimation#closeAnimationName")
        local unloadAnimationSpeed = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.baler.baleAnimation#unloadAnimationSpeed"), 1)
        local closeAnimationSpeed = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.baler.baleAnimation#closeAnimationSpeed"), 1)
        if unloadAnimationName ~= nil and closeAnimationName ~= nil then
            if self.playAnimation ~= nil and self.animations ~= nil then
                if self.animations[unloadAnimationName] ~= nil and self.animations[closeAnimationName] ~= nil then
                    --print("has unload animation")
                    self.cottonBaler.baleUnloadAnimationName = unloadAnimationName
                    self.cottonBaler.baleUnloadAnimationSpeed = unloadAnimationSpeed

                    self.cottonBaler.baleCloseAnimationName = closeAnimationName
                    self.cottonBaler.baleCloseAnimationSpeed = closeAnimationSpeed

                    self.cottonBaler.baleDropAnimTime = getXMLFloat(self.xmlFile, "vehicle.baler.baleAnimation#baleDropAnimTime")
                    if self.cottonBaler.baleDropAnimTime == nil then
                        self.cottonBaler.baleDropAnimTime = self:getAnimationDuration(self.cottonBaler.baleUnloadAnimationName)
                    else
                        self.cottonBaler.baleDropAnimTime = self.cottonBaler.baleDropAnimTime * 1000
                    end
                else
                    print("Error: Failed to find unload animations '"..unloadAnimationName.."' and '"..closeAnimationName.."' in '"..self.configFileName.."'.")
                end
            else
                print("Error: There is an unload animation in '"..self.configFileName.."' but it is not a AnimatedVehicle. Change to a vehicle type which has the AnimatedVehicle specialization.")
            end
        end
    end

    self.cottonBaler.baleTypes = {}
    local i = 0
    while true do
        local key = string.format("vehicle.baler.baleTypes.baleType(%d)", i)
        if not hasXMLProperty(self.xmlFile, key) then
            break
        end
        local isRoundBale = Utils.getNoNil(getXMLBool(self.xmlFile, key.."#isRoundBale"), false)
        local width = Utils.round(Utils.getNoNil(getXMLFloat(self.xmlFile, key.."#width"), 1.2), 2)
        local height = Utils.round(Utils.getNoNil(getXMLFloat(self.xmlFile, key.."#height"), 0.9), 2)
        local length = Utils.round(Utils.getNoNil(getXMLFloat(self.xmlFile, key.."#length"), 2.4), 2)
        local diameter = Utils.round(Utils.getNoNil(getXMLFloat(self.xmlFile, key.."#diameter"), 1.8), 2)
        table.insert(self.cottonBaler.baleTypes, {isRoundBale=isRoundBale, width=width, height=height, length=length, diameter=diameter})
        --print("debug "..tostring(i).."  "..tostring(isRoundBale).."  "..tostring(width).."  "..tostring(height).."  "..tostring(length).."  "..tostring(diameter))
        i = i + 1
    end
    self.cottonBaler.currentBaleTypeId = 1

    if table.getn(self.cottonBaler.baleTypes) == 0 then
        self.cottonBaler.baleTypes = nil
    end

    if self.isClient then
        self.cottonBaler.sampleBaler = SoundUtil.loadSample(self.xmlFile, {}, "vehicle.baler.balerSound", nil, self.baseDirectory)
        self.cottonBaler.sampleBalerAlarm = SoundUtil.loadSample(self.xmlFile, {}, "vehicle.baler.balerAlarm", nil, self.baseDirectory)
        self.cottonBaler.sampleBalerEject = SoundUtil.loadSample(self.xmlFile, {}, "vehicle.baler.balerBaleEject", nil, self.baseDirectory)
        self.cottonBaler.sampleBalerDoor = SoundUtil.loadSample(self.xmlFile, {}, "vehicle.baler.balerDoor", nil, self.baseDirectory)
        self.cottonBaler.sampleBalerKnotCleaning = SoundUtil.loadSample(self.xmlFile, {}, "vehicle.baler.balerKnotCleaning", nil, self.baseDirectory)
        self.cottonBaler.knotCleaningTime = 10000

        self.cottonBaler.uvScrollParts = Utils.loadScrollers(self.components, self.xmlFile, "vehicle.baler.uvScrollParts.uvScrollPart", {}, false)
        self.cottonBaler.turnedOnRotationNodes = Utils.loadRotationNodes(self.xmlFile, {}, "vehicle.turnedOnRotationNodes.turnedOnRotationNode", "baler", self.components)

        self.cottonBaler.fillEffects = EffectManager:loadEffect(self.xmlFile, "vehicle.baler.fillEffect", self.components, self)

        self.cottonBaler.knotingAnimation = getXMLString(self.xmlFile, "vehicle.baler.knotingAnimation#name")
        self.cottonBaler.knotingAnimationSpeed = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.knotingAnimation#speed"), 1)
    end

    self.cottonBaler.balingAnimationName = Utils.getNoNil(getXMLString(self.xmlFile, "vehicle.baler.balingAnimation#name"), "")
    if self.playAnimation == nil or self.getIsAnimationPlaying == nil then
        self.cottonBaler.balingAnimationName = ""
    end

    self.cottonBaler.lastAreaBiggerZero = false
    self.cottonBaler.lastAreaBiggerZeroSent = false
    self.cottonBaler.lastAreaBiggerZeroTime = 0

    self.cottonBaler.unloadingState = cottonBaler.UNLOADING_CLOSED
    self.cottonBaler.pickupFillTypes = {}

    self.cottonBaler.bales = {}
    self.cottonBaler.hasBaler = true

    self.cottonBaler.dummyBale = {}
    self.cottonBaler.dummyBale.scaleNode = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.baler.baleAnimation#scaleNode"))
    self.cottonBaler.dummyBale.baleNode = Utils.indexToObject(self.components, getXMLString(self.xmlFile, "vehicle.baler.baleAnimation#baleNode"))
    self.cottonBaler.dummyBale.currentBaleFillType = FillUtil.FILLTYPE_UNKNOWN
    self.cottonBaler.dummyBale.currentBale = nil

    self.cottonBaler.allowsBaleUnloading = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.baler.baleUnloading#allowed"), false)
    self.cottonBaler.baleUnloadingTime = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.baler.baleUnloading#time"), 4) * 1000
    self.cottonBaler.baleFoldThreshold = Utils.getNoNil(getXMLFloat(self.xmlFile, "vehicle.baler.baleUnloading#foldThreshold"), 0.25) * self:getCapacity()

    self.cottonBaler.isBaleUnloading = false
    self.cottonBaler.isSpeedLimitActive = false

    self.cottonBaler.dirtyFlag = self:getNextDirtyFlag()
end

function cottonBaler:postLoad(savegame)

    for fillType, enabled in pairs(self:getUnitFillTypes(self.cottonBaler.fillUnitIndex)) do
        if enabled and fillType ~= FillUtil.FILLTYPE_UNKNOWN then
            if FruitUtil.fillTypeIsWindrow[fillType] then
                table.insert(self.cottonBaler.pickupFillTypes, fillType)
            end
        end
    end

    if self.isClient then
        self.cottonBaler.fillParticleSystems = {}
        self.cottonBaler.currentFillParticleSystem = nil
        local i = 0
        while true do
            local key = string.format("vehicle.baler.fillParticleSystems.emitterShape(%d)", i)
            if not hasXMLProperty(self.xmlFile, key) then
                break
            end

            local emitterShape = Utils.indexToObject(self.components, getXMLString(self.xmlFile, key.."#node"))
            local particleType = getXMLString(self.xmlFile, key.."#particleType")
            if emitterShape ~= nil then
                for _, fillType in pairs(self.cottonBaler.pickupFillTypes) do
                    local particleSystem = MaterialUtil.getParticleSystem(fillType, particleType)
                    if particleSystem ~= nil then
                        if self.cottonBaler.fillParticleSystems[fillType] == nil then
                            self.cottonBaler.fillParticleSystems[fillType] = {}
                        end
                        table.insert(self.cottonBaler.fillParticleSystems[fillType], ParticleUtil.copyParticleSystem(self.xmlFile, key, particleSystem, emitterShape))
                    end
                end
            end
            i = i + 1
        end
    end

    if savegame ~= nil and not savegame.resetVehicles then
        local numBales = getXMLInt(savegame.xmlFile, savegame.key.."#numBales")
        if numBales ~= nil then
            self.cottonBaler.balesToLoad = {}
            for i=1, numBales do
                local baleKey = savegame.key..string.format(".bale(%d)", i-1)
                local bale = {}
                local fillTypeStr = getXMLString(savegame.xmlFile, baleKey.."#fillType")
                local fillType = FillUtil.fillTypeNameToInt[fillTypeStr]
                bale.fillType = fillType
                bale.fillLevel = getXMLFloat(savegame.xmlFile, baleKey.."#fillLevel")
                bale.baleTime = getXMLFloat(savegame.xmlFile, baleKey.."#baleTime")
                table.insert(self.cottonBaler.balesToLoad, bale)
            end
        end
    end

    -- we need always empty combine fillUnit to working proper.
    self:setUnitFillLevel(self.overloading.fillUnitIndex, 0, FillUtil.FILLTYPE_UNKNOWN, true);
    
end

function cottonBaler:delete()
    for k, _ in pairs(self.cottonBaler.bales) do
        self:dropBale(k)
    end

    if self.cottonBaler.dummyBale.currentBale ~= nil then
        delete(self.cottonBaler.dummyBale.currentBale)
        self.cottonBaler.dummyBale.currentBale = nil
    end

    if self.isClient then
        SoundUtil.deleteSample(self.cottonBaler.sampleBaler)
        SoundUtil.deleteSample(self.cottonBaler.sampleBalerAlarm)
        SoundUtil.deleteSample(self.cottonBaler.sampleBalerDoor)
        SoundUtil.deleteSample(self.cottonBaler.sampleBalerEject)
        SoundUtil.deleteSample(self.cottonBaler.sampleBalerKnotCleaning)

        for _,v in pairs(self.cottonBaler.fillParticleSystems) do
            ParticleUtil.deleteParticleSystems(v)
        end

        EffectManager:deleteEffects(self.cottonBaler.fillEffects);
    end
end

function cottonBaler:readStream(streamId, connection)
    if self.cottonBaler.baleUnloadAnimationName ~= nil then
        local state = streamReadUIntN(streamId, 7)
        local animTime = streamReadFloat32(streamId)
        if state == cottonBaler.UNLOADING_CLOSED or state == cottonBaler.UNLOADING_CLOSING  then
            self:setIsUnloadingBale(false, true)
            self:setRealAnimationTime(self.cottonBaler.baleCloseAnimationName, animTime)
        elseif state == cottonBaler.UNLOADING_OPEN or state == cottonBaler.UNLOADING_OPENING then
            self:setIsUnloadingBale(true, true)
            self:setRealAnimationTime(self.cottonBaler.baleUnloadAnimationName, animTime)
        end
    end

    local numBales = streamReadUInt8(streamId)
    for i=1, numBales do
        local fillType = streamReadInt8(streamId)
        local fillLevel = streamReadFloat32(streamId)
        self:createBale(fillType, fillLevel)
        if self.cottonBaler.baleAnimCurve ~= nil then
            local baleTime = streamReadFloat32(streamId)
            self:setBaleTime(i, baleTime)
        end
    end

    self.cottonBaler.lastAreaBiggerZero = streamReadBool(streamId)
end

function cottonBaler:writeStream(streamId, connection)

    if self.cottonBaler.baleUnloadAnimationName ~= nil then
        streamWriteUIntN(streamId, self.cottonBaler.unloadingState, 7)
        local animTime = 0
        if self.cottonBaler.unloadingState == cottonBaler.UNLOADING_CLOSED or self.cottonBaler.unloadingState == cottonBaler.UNLOADING_CLOSING  then
            animTime = self:getRealAnimationTime(self.cottonBaler.baleCloseAnimationName)
        elseif self.cottonBaler.unloadingState == cottonBaler.UNLOADING_OPEN or self.cottonBaler.unloadingState == cottonBaler.UNLOADING_OPENING then
            animTime = self:getRealAnimationTime(self.cottonBaler.baleUnloadAnimationName)
        end
        streamWriteFloat32(streamId, animTime)
    end

    streamWriteUInt8(streamId, table.getn(self.cottonBaler.bales))
    for i=1, table.getn(self.cottonBaler.bales) do
        local bale = self.cottonBaler.bales[i]
        streamWriteInt8(streamId, bale.fillType)
        streamWriteFloat32(streamId, bale.fillLevel)
        if self.cottonBaler.baleAnimCurve ~= nil then
            streamWriteFloat32(streamId, bale.time)
        end
    end

    streamWriteBool(streamId, self.cottonBaler.lastAreaBiggerZero)
end

function cottonBaler:readUpdateStream(streamId, timestamp, connection)
    if connection:getIsServer() then
        self.cottonBaler.lastAreaBiggerZero = streamReadBool(streamId)
    end
end

function cottonBaler:writeUpdateStream(streamId, connection, dirtyMask)
    if not connection:getIsServer() then
        streamWriteBool(streamId, self.cottonBaler.lastAreaBiggerZero)
    end
end

function cottonBaler:getSaveAttributesAndNodes(nodeIdent)

    local attributes = 'numBales="'..table.getn(self.cottonBaler.bales)..'"'
    local nodes = ""
    local baleNum = 0

    for i=1, table.getn(self.cottonBaler.bales) do
        local bale = self.cottonBaler.bales[i]
        local fillTypeStr = "unknown"
        if bale.fillType ~= FillUtil.FILLTYPE_UNKNOWN then
            fillTypeStr = FillUtil.fillTypeIntToName[bale.fillType]
        end

        if baleNum>0 then
            nodes = nodes.."\n"
        end
        nodes = nodes..nodeIdent..'<bale fillType="'..fillTypeStr..'" fillLevel="'..bale.fillLevel..'"'
        if self.cottonBaler.baleAnimCurve ~= nil then
            nodes = nodes..' baleTime="'..bale.time..'"'
        end
        nodes = nodes..' />'
        baleNum = baleNum+1
    end
    return attributes,nodes
end

function cottonBaler:mouseEvent(posX, posY, isDown, isUp, button)
end

function cottonBaler:keyEvent(unicode, sym, modifier, isDown)
end

function cottonBaler:update(dt)
    
    if self.firstTimeRun and self.cottonBaler.balesToLoad ~= nil then
        if table.getn(self.cottonBaler.balesToLoad) > 0 then
            local v = self.cottonBaler.balesToLoad[1];

            if v.targetBaleTime == nil then
                self:createBale(v.fillType, v.fillLevel)
                self:setBaleTime(table.getn(self.cottonBaler.bales), 0, true);
                v.targetBaleTime = v.baleTime;
                v.baleTime = 0;
            else
                v.baleTime = math.min(v.baleTime + dt / 1000, v.targetBaleTime);
                self:setBaleTime(table.getn(self.cottonBaler.bales), v.baleTime, true);

                if v.baleTime == v.targetBaleTime then

                    local index = table.getn(self.cottonBaler.balesToLoad);
                    if index == 1 then
                        self.cottonBaler.balesToLoad = nil;
                    else
                        table.remove(self.cottonBaler.balesToLoad, 1);
                    end
                end
            end
        end
    end

    -- fcelsa controllare lo scarico automatico della balla...
    if self:isUnloadingAllowed() then
        if self.cottonBaler.baleUnloadAnimationName ~= nil or self.cottonBaler.allowsBaleUnloading then
            if self.cottonBaler.unloadingState == cottonBaler.UNLOADING_CLOSED then
                if table.getn(self.cottonBaler.bales) > 0 then
                    self:setIsUnloadingBale(true);
                end
            elseif self.cottonBaler.unloadingState == cottonBaler.UNLOADING_OPEN then
                if self.cottonBaler.baleUnloadAnimationName ~= nil then
                    self:setIsUnloadingBale(false);
                end
            end
        end
    end

    if self.isClient then
        -- fcelsa: turnedOnRotationNodes work from combine
        -- Utils.updateRotationNodes(self, self.cottonBaler.turnedOnRotationNodes, dt, self:getIsActive() and self:getIsTurnedOn() )
        Utils.updateScrollers(self.cottonBaler.uvScrollParts, dt, self:getIsActive() and self:getIsTurnedOn());
    end
    
end

function cottonBaler:updateTick(dt)
    
    if self.isServer then
        self.cottonBaler.lastAreaBiggerZero = false
    end

    if self:getIsActive() then
        if self:getIsTurnedOn() then
            if self:allowPickingUp() and self.isStrawEnabled == false then
                if g_currentMission:getCanAddLimitedObject(FSBaseMission.LIMITED_OBJECT_TYPE_BALE) then
                    --self.cottonBaler.isSpeedLimitActive = true
                    if self.isServer then
                        --local workAreas, _, _ = self:getTypedNetworkAreas(WorkArea.AREATYPE_BALER, false)
                        local totalLiters = 0
                        local usedFillType = FillUtil.FILLTYPE_UNKNOWN

                        -- cotton_windrow it's hardcoded here... to do better implementation
                        -- local usedFillType = FillUtil.fillTypeNameToInt["cotton"];

                        -- fruitType, windrowName, windrowNameI18N, category, windrowPricePerLiter, windrowLiterPerSqm, 
                        -- windrowHudOverlayFilename, windrowHudOverlayFilenameSmall, massPerLiter

                        local fruitDesc = FruitUtil.fruitIndexToDesc[self:getUnitLastValidFillType(self.overloading.fillUnitIndex)];
                        if fruitDesc ~= nil and fruitDesc.windrowLiterPerSqm ~= nil and fruitDesc.windrowName == "straw" then
                            local windrowFillType = FruitUtil.fruitTypeToWindrowFillType[fruitDesc.index];
                            usedFillType = windrowFillType
                            --renderText(0.3, 0.05, 0.015, ":: " .. fruitDesc.name .. "  " .. fruitDesc.windrowName);
                        else
                            usedFillType = self:getUnitFillType(self.overloading.fillUnitIndex)
                        end
                        
                        --fcelsa original 
                        --if table.getn(workAreas) > 0 then
                        --    totalLiters, usedFillType = self:processBalerAreas(workAreas, self.cottonBaler.pickupFillTypes)
                        --end

                        local fillLevel = self:getUnitFillLevel(self.overloading.fillUnitIndex);
                        local capacity = self:getUnitCapacity(self.overloading.fillUnitIndex);
                        if fillLevel == capacity then
                            self:setUnitFillLevel(self.overloading.fillUnitIndex, 1, self:getUnitFillType(self.overloading.fillUnitIndex), true);
                        end
                        if fillLevel >= (0.65 * capacity) then
                            totalLiters = fillLevel
                            self:setUnitFillLevel(self.overloading.fillUnitIndex, 1, self:getUnitFillType(self.overloading.fillUnitIndex), true);
                        end

                        if totalLiters > 0 then
                            self.cottonBaler.lastAreaBiggerZero = true
                            if self.cottonBaler.lastAreaBiggerZero ~= self.cottonBaler.lastAreaBiggerZeroSent then
                                self:raiseDirtyFlags(self.cottonBaler.dirtyFlag)
                                self.cottonBaler.lastAreaBiggerZeroSent = self.cottonBaler.lastAreaBiggerZero
                            end

                            local deltaLevel = totalLiters * self.cottonBaler.fillScale

                            if self.cottonBaler.baleUnloadAnimationName == nil then
                                -- move all bales
                                local deltaTime = self:getTimeFromLevel(deltaLevel)
                                self:moveBales(deltaTime)
                            end

                            local oldFillLevel = self:getUnitFillLevel(self.cottonBaler.fillUnitIndex)
                            self:setUnitFillLevel(self.cottonBaler.fillUnitIndex, oldFillLevel+deltaLevel, usedFillType, true)
                            if self:getUnitFillLevel(self.cottonBaler.fillUnitIndex) >= self:getUnitCapacity(self.cottonBaler.fillUnitIndex) then
                                if self.cottonBaler.baleTypes ~= nil then
                                    -- create bale
                                    if self.cottonBaler.baleAnimCurve ~= nil then
                                        local restDeltaFillLevel = deltaLevel - (self:getUnitFillLevel(self.cottonBaler.fillUnitIndex)-oldFillLevel)
                                        self:setUnitFillLevel(self.cottonBaler.fillUnitIndex, restDeltaFillLevel, usedFillType, true)

                                        self:createBale(usedFillType, self:getUnitCapacity(self.cottonBaler.fillUnitIndex))

                                        local numBales = table.getn(self.cottonBaler.bales)
                                        local bale = self.cottonBaler.bales[numBales]

                                        self:moveBale(numBales, self:getTimeFromLevel(restDeltaFillLevel), true)
                                        -- note: self.baler.bales[numBales] can not be accessed anymore since the bale might be dropped already
                                        g_server:broadcastEvent(cottonBalerCreateBaleEvent:new(self, usedFillType), nil, nil, self)
                                    elseif self.cottonBaler.baleUnloadAnimationName ~= nil then

                                        self:createBale(usedFillType, self:getUnitCapacity(self.cottonBaler.fillUnitIndex))
                                        g_server:broadcastEvent(cottonBalerCreateBaleEvent:new(self, usedFillType), nil, nil, self)
                                    end
                                end
                            end
                        end
                    end
                else
                    g_currentMission:showBlinkingWarning(g_i18n:getText("warning_tooManyBales"), 500)
                end
            else
                local fillLevel = self:getUnitFillLevel(self.overloading.fillUnitIndex);
                local capacity = self:getUnitCapacity(self.overloading.fillUnitIndex);
                if fillLevel >= (0.65 * capacity) then   
                    self:setUnitFillLevel(self.overloading.fillUnitIndex, 1, self:getUnitFillType(self.overloading.fillUnitIndex), true);
                end
            end

            if self.cottonBaler.lastAreaBiggerZero and self.fillUnits[self.cottonBaler.fillUnitIndex].lastValidFillType ~= FillUtil.FILLTYPE_UNKNOWN then
                self.cottonBaler.lastAreaBiggerZeroTime = 500
            else
                if self.cottonBaler.lastAreaBiggerZeroTime > 0 then
                    self.cottonBaler.lastAreaBiggerZeroTime = self.cottonBaler.lastAreaBiggerZeroTime - dt
                end
            end

            if self.isClient then
                if self.cottonBaler.fillEffects ~= nil then
                    if self.cottonBaler.lastAreaBiggerZeroTime > 0 then
                        EffectManager:setFillType(self.cottonBaler.fillEffects, self.fillUnits[self.cottonBaler.fillUnitIndex].lastValidFillType)
                        EffectManager:startEffects(self.cottonBaler.fillEffects)
                    else
                        EffectManager:stopEffects(self.cottonBaler.fillEffects)
                    end
                end

                local currentFillParticleSystem = self.cottonBaler.fillParticleSystems[self.fillUnits[self.cottonBaler.fillUnitIndex].lastValidFillType]
                if currentFillParticleSystem ~= self.cottonBaler.currentFillParticleSystem then
                    if self.cottonBaler.currentFillParticleSystem ~= nil then
                        for _, ps in pairs(self.cottonBaler.currentFillParticleSystem) do
                            ParticleUtil.setEmittingState(ps, false)
                        end
                        self.cottonBaler.currentFillParticleSystem = nil
                    end
                    self.cottonBaler.currentFillParticleSystem = currentFillParticleSystem
                end

                if self.cottonBaler.currentFillParticleSystem ~= nil then
                    for _, ps in pairs(self.cottonBaler.currentFillParticleSystem) do
                        ParticleUtil.setEmittingState(ps, self.cottonBaler.lastAreaBiggerZeroTime > 0)
                    end
                end

                if self:getIsActiveForSound() then
                    if self.cottonBaler.knotCleaningTime <= g_currentMission.time then
                        SoundUtil.playSample(self.cottonBaler.sampleBalerKnotCleaning, 1, 0, nil)
                        self.cottonBaler.knotCleaningTime = g_currentMission.time + 120000
                    end
                    SoundUtil.playSample(self.cottonBaler.sampleBaler, 0, 0, nil)
                end
            end
        else
            if self.cottonBaler.isBaleUnloading and self.isServer then
                local deltaTime = dt / self.cottonBaler.baleUnloadingTime
                self:moveBales(deltaTime)
                self.cottonBaler.isSpeedLimitActive = true
            else
                self.cottonBaler.isSpeedLimitActive = false
            end
        end

        if self.isClient then
            if not self:getIsTurnedOn() then
                SoundUtil.stopSample(self.cottonBaler.sampleBalerKnotCleaning)
                SoundUtil.stopSample(self.cottonBaler.sampleBaler)
            end

            if self:getIsTurnedOn() and self:getUnitFillLevel(self.cottonBaler.fillUnitIndex) > (self:getUnitCapacity(self.cottonBaler.fillUnitIndex) * 0.95) and self:getUnitFillLevel(self.cottonBaler.fillUnitIndex) < self:getUnitCapacity(self.cottonBaler.fillUnitIndex) then
                -- start alarm sound
                if self:getIsActiveForSound() then
                    SoundUtil.playSample(self.cottonBaler.sampleBalerAlarm, 0, 0, nil)
                end
            else
                SoundUtil.stopSample(self.cottonBaler.sampleBalerAlarm)
            end

            --delete dummy bale on client after physical bale is displayed
            if self.cottonBaler.unloadingState == cottonBaler.UNLOADING_OPEN then
                if getNumOfChildren(self.cottonBaler.baleAnimRoot) > 0 then
                    delete(getChildAt(self.cottonBaler.baleAnimRoot, 0));
                end;
            end;
        end;

        if self.cottonBaler.unloadingState == cottonBaler.UNLOADING_OPENING then
            local isPlaying = self:getIsAnimationPlaying(self.cottonBaler.baleUnloadAnimationName)
            local animTime = self:getRealAnimationTime(self.cottonBaler.baleUnloadAnimationName)
            if not isPlaying or animTime >= self.cottonBaler.baleDropAnimTime then
                if table.getn(self.cottonBaler.bales) > 0 then
                    self:dropBale(1)
                    if self.isServer then
                        self:setUnitFillLevel(self.cottonBaler.fillUnitIndex, 0, self:getUnitFillType(self.cottonBaler.fillUnitIndex), true)
                    end
                end
                if not isPlaying then
                    self.cottonBaler.unloadingState = cottonBaler.UNLOADING_OPEN

                    if self.isClient then
                        SoundUtil.stopSample(self.cottonBaler.sampleBalerEject)
                        SoundUtil.stopSample(self.cottonBaler.sampleBalerDoor)
                    end
                end
            end
        elseif self.cottonBaler.unloadingState == cottonBaler.UNLOADING_CLOSING then
            if not self:getIsAnimationPlaying(self.cottonBaler.baleCloseAnimationName) then
                self.cottonBaler.unloadingState = cottonBaler.UNLOADING_CLOSED
                if self.isClient then
                    SoundUtil.stopSample(self.cottonBaler.sampleBalerDoor)
                end
            end
        end
    end
end

---Called on draw
-- @includeCode
function cottonBaler:draw()
    if self.isClient then
        if self:getIsActiveForInput(true) then
            if self:isUnloadingAllowed() then
                if self.cottonBaler.baleUnloadAnimationName ~= nil or self.cottonBaler.allowsBaleUnloading then
                    if self.cottonBaler.unloadingState == cottonBaler.UNLOADING_CLOSED then
                        if table.getn(self.cottonBaler.bales) > 0 then
                            g_currentMission:addHelpButtonText(g_i18n:getText("action_unloadBaler"), InputBinding.IMPLEMENT_EXTRA3, nil, GS_PRIO_HIGH)
                        end
                    elseif self.cottonBaler.unloadingState == cottonBaler.UNLOADING_OPEN then
                        if self.cottonBaler.baleUnloadAnimationName ~= nil then
                            g_currentMission:addHelpButtonText(g_i18n:getText("action_closeBack"), InputBinding.IMPLEMENT_EXTRA3, nil, GS_PRIO_HIGH)
                        end
                    end
                end
            end
        end
    end
end

--function cottonBaler:getIsFoldAllowed(superFunc, onAiTurnOn)
--    if (table.getn(self.cottonBaler.bales) > 0 and self:getUnitFillLevel(self.cottonBaler.fillUnitIndex) > self.cottonBaler.baleFoldThreshold) or table.getn(self.cottonBaler.bales) > 1 or self:getIsTurnedOn() then
--        return false
--    end
--
--    if superFunc ~= nil then
--        return superFunc(self, onAiTurnOn)
--    end
--
--    return true
--end

function cottonBaler:onDeactivate()
    if self.cottonBaler.balingAnimationName ~= "" then
        self:stopAnimation(self.cottonBaler.balingAnimationName, true)
    end
    if self.isClient then
        if self.cottonBaler.fillEffects ~= nil then
            EffectManager:stopEffects(self.cottonBaler.fillEffects)
        end
        if self.cottonBaler.currentFillParticleSystem ~= nil then
            for _, ps in pairs(self.cottonBaler.currentFillParticleSystem) do
                ParticleUtil.setEmittingState(ps, false)
            end
        end
    end
end

function cottonBaler:onDeactivateSounds()
    if self.isClient then
        SoundUtil.stopSample(self.cottonBaler.sampleBaler, true)
        SoundUtil.stopSample(self.cottonBaler.sampleBalerAlarm, true)
        SoundUtil.stopSample(self.cottonBaler.sampleBalerDoor, true)
        SoundUtil.stopSample(self.cottonBaler.sampleBalerEject, true)
        SoundUtil.stopSample(self.cottonBaler.sampleBalerKnotCleaning, true)
    end
end

function cottonBaler:setUnitFillLevel(fillUnitIndex, fillLevel, fillType, force, fillInfo)
    if fillUnitIndex == self.cottonBaler.fillUnitIndex then
        if self.cottonBaler.dummyBale.baleNode ~= nil and fillLevel > 0 and fillLevel < self:getUnitCapacity(fillUnitIndex) and (self.cottonBaler.dummyBale.currentBale == nil or self.cottonBaler.dummyBale.currentBaleFillType ~= fillType) then
            if self.cottonBaler.dummyBale.currentBale ~= nil then
                delete(self.cottonBaler.dummyBale.currentBale)
                self.cottonBaler.dummyBale.currentBale = nil
            end
            local t = self.cottonBaler.baleTypes[self.cottonBaler.currentBaleTypeId]

            local baleType = BaleUtil.getBale(fillType, t.width, t.height, t.length, t.diameter, t.isRoundBale)

            local baleRoot = Utils.loadSharedI3DFile(baleType.filename, self.modDir, false, false)
            local baleId = getChildAt(baleRoot, 0)
            setRigidBodyType(baleId, "NoRigidBody")
            link(self.cottonBaler.dummyBale.baleNode, baleId)
            delete(baleRoot)
            self.cottonBaler.dummyBale.currentBale = baleId
            self.cottonBaler.dummyBale.currentBaleFillType = fillType
        end

        if self.cottonBaler.dummyBale.currentBale ~= nil then
            local percent = fillLevel / self:getUnitCapacity(fillUnitIndex)
            local y = 1
            if getUserAttribute(self.cottonBaler.dummyBale.currentBale, "isRoundbale") then
                y = percent
            end
            setScale(self.cottonBaler.dummyBale.scaleNode, 1, y, percent)
        end
    end
end

function cottonBaler:onTurnedOn(noEventSend)
    if self.setFoldState ~= nil then
        self:setFoldState(-1)
    end
    if self.cottonBaler.balingAnimationName ~= "" then
        self:playAnimation(self.cottonBaler.balingAnimationName, 1, self:getAnimationTime(self.cottonBaler.balingAnimationName), true)
    end
    -- always straw disabled 
    self:setIsStrawEnabled(false);
end

function cottonBaler:onTurnedOff(noEventSend)

    if self.cottonBaler.balingAnimationName ~= "" then
        self:stopAnimation(self.cottonBaler.balingAnimationName, true)
    end
    if self.isClient then
        if self.cottonBaler.fillEffects ~= nil then
            EffectManager:stopEffects(self.cottonBaler.fillEffects)
        end
        if self.cottonBaler.currentFillParticleSystem ~= nil then
            for _, ps in pairs(self.cottonBaler.currentFillParticleSystem) do
                ParticleUtil.setEmittingState(ps, false)
            end
        end
    end
end

function cottonBaler:doCheckSpeedLimit(superFunc)
    local parent = false
    if superFunc ~= nil then
        parent = superFunc(self)
    end

    return parent or self.cottonBaler.isSpeedLimitActive
end

function cottonBaler:isUnloadingAllowed()
    return not self.cottonBaler.allowsBaleUnloading or (self.cottonBaler.allowsBaleUnloading and not self:getIsTurnedOn() and not self.cottonBaler.isBaleUnloading);
end

function cottonBaler:setIsUnloadingBale(isUnloadingBale, noEventSend)
    if self.cottonBaler.baleUnloadAnimationName ~= nil then
        if isUnloadingBale then
            if self.cottonBaler.unloadingState ~= cottonBaler.UNLOADING_OPENING then
                cottonBalerSetIsUnloadingBaleEvent.sendEvent(self, isUnloadingBale, noEventSend)
                self.cottonBaler.unloadingState = cottonBaler.UNLOADING_OPENING
                if self.isClient and self:getIsActiveForSound() then
                    SoundUtil.playSample(self.cottonBaler.sampleBalerEject, 1, 0, nil)
                    SoundUtil.playSample(self.cottonBaler.sampleBalerDoor, 1, 0, nil)
                end
                self:playAnimation(self.cottonBaler.baleUnloadAnimationName, self.cottonBaler.baleUnloadAnimationSpeed, nil, true)
            end
        else
            if self.cottonBaler.unloadingState ~= cottonBaler.UNLOADING_CLOSING then
                cottonBalerSetIsUnloadingBaleEvent.sendEvent(self, isUnloadingBale, noEventSend)
                self.cottonBaler.unloadingState = cottonBaler.UNLOADING_CLOSING
                if self.isClient and self:getIsActiveForSound() then
                    SoundUtil.playSample(self.cottonBaler.sampleBalerDoor, 1, 0, nil)
                end
                self:playAnimation(self.cottonBaler.baleCloseAnimationName, self.cottonBaler.baleCloseAnimationSpeed, nil, true)
            end
        end
    elseif self.cottonBaler.allowsBaleUnloading then
        if isUnloadingBale then
            cottonBalerSetIsUnloadingBaleEvent.sendEvent(self, isUnloadingBale, noEventSend)
            self.cottonBaler.isBaleUnloading = true
        end
    end
end

function cottonBaler:getTimeFromLevel(level)
    -- level = capacity -> time = firstBaleMarker
    -- level = 0           -> time = 0
    if self.cottonBaler.firstBaleMarker ~= nil then
        return level / self:getCapacity() * self.cottonBaler.firstBaleMarker
    end
    return 0
end

function cottonBaler:moveBales(dt)
    for i=table.getn(self.cottonBaler.bales), 1, -1 do
        self:moveBale(i, dt)
    end
end

function cottonBaler:moveBale(i, dt, noEventSend)
    local bale = self.cottonBaler.bales[i]
    self:setBaleTime(i, bale.time + dt, noEventSend)
end

function cottonBaler:setBaleTime(i, baleTime, noEventSend)
    if self.cottonBaler.baleAnimCurve ~= nil then
        local bale = self.cottonBaler.bales[i]
        bale.time = baleTime
        if self.isServer then
            local v = self.cottonBaler.baleAnimCurve:get(bale.time)
            --setTranslation(bale.id, v[1], v[2], v[3])
            --setRotation(bale.id, v[4], v[5], v[6])
            setTranslation(bale.baleJointNode, v[1], v[2], v[3])
            setRotation(bale.baleJointNode, v[4], v[5], v[6])
            if bale.baleJointIndex ~= 0 then
                setJointFrame(bale.baleJointIndex, 0, bale.baleJointNode)
            end
        end
        if bale.time >= 1 then
            self:dropBale(i)
        end
        if table.getn(self.cottonBaler.bales) == 0 then
            self.cottonBaler.isBaleUnloading = false
        end
        if self.isServer then
            if noEventSend == nil or not noEventSend then
                g_server:broadcastEvent(cottonBalerSetBaleTimeEvent:new(self, i, bale.time), nil, nil, self)
            end
        end
    end
end

function cottonBaler:allowFillType(fillType)
    return true
    --return self.cottonBaler.pickupFillTypes[fillType] ~= nil
end


-- fcelsa no superfunc due to missing pickup spec.
function cottonBaler:allowPickingUp()
    if self.cottonBaler.baleUnloadAnimationName ~= nil then
        if table.getn(self.cottonBaler.bales) > 0 or self.cottonBaler.unloadingState ~= cottonBaler.UNLOADING_CLOSED then
            return false;
        end
    end

    if self.combineIsFilling then
        return true;
    end

    return false;
end

---Create new bale in baler
function cottonBaler:createBale(baleFillType, fillLevel)

    if self.cottonBaler.knotingAnimation ~= nil then
        self:playAnimation(self.cottonBaler.knotingAnimation, self.cottonBaler.knotingAnimationSpeed, nil, true)
    end

    if self.cottonBaler.dummyBale.currentBale ~= nil then
        delete(self.cottonBaler.dummyBale.currentBale)
        self.cottonBaler.dummyBale.currentBale = nil
    end

    local t = self.cottonBaler.baleTypes[self.cottonBaler.currentBaleTypeId]
    local baleType = BaleUtil.getBale(baleFillType, t.width, t.height, t.length, t.diameter, t.isRoundBale)

    local bale = {}
    bale.filename = Utils.getFilename(baleType.filename, "")
    bale.time = 0
    bale.fillType = baleFillType
    bale.fillLevel = fillLevel

    if self.cottonBaler.baleUnloadAnimationName ~= nil then
        local baleRoot = Utils.loadSharedI3DFile(baleType.filename, self.modDir, false, false)
        local baleId = getChildAt(baleRoot, 0)
        link(self.cottonBaler.baleAnimRoot, baleId)
        delete(baleRoot)
        bale.id = baleId
    end

    if self.isServer and self.cottonBaler.baleUnloadAnimationName == nil then
        local x,y,z = getWorldTranslation(self.cottonBaler.baleAnimRoot)
        local rx,ry,rz = getWorldRotation(self.cottonBaler.baleAnimRoot)

        local baleObject = Bale:new(self.isServer, self.isClient)
        baleObject:load(bale.filename, x,y,z,rx,ry,rz, bale.fillLevel)
        baleObject:register()

        local baleJointNode = createTransformGroup("BaleJointTG")
        link(self.cottonBaler.baleAnimRoot, baleJointNode)
        setTranslation(baleJointNode, 0,0,0)
        setRotation(baleJointNode, 0,0,0)

        local constr = JointConstructor:new()
        constr:setActors(self.cottonBaler.baleAnimRootComponent, baleObject.nodeId)
        constr:setJointTransforms(baleJointNode, baleObject.nodeId)
        for i=1, 3 do
            constr:setRotationLimit(i-1, 0, 0)
            constr:setTranslationLimit(i-1, true, 0, 0)
        end
        constr:setEnableCollision(false)
        local baleJointIndex = constr:finalize()

        g_currentMission:removeItemToSave(baleObject)

        bale.baleJointNode = baleJointNode
        bale.baleJointIndex = baleJointIndex
        bale.baleObject = baleObject
    end

    table.insert(self.cottonBaler.bales, bale)
end

function cottonBaler:dropBale(baleIndex)
    local bale = self.cottonBaler.bales[baleIndex]

    if self.isServer then
        local baleObject

        if bale.baleJointIndex ~= nil then
            baleObject = bale.baleObject
            removeJoint(bale.baleJointIndex)
            delete(bale.baleJointNode)
            g_currentMission:addItemToSave(bale.baleObject)
        else
            baleObject = Bale:new(self.isServer, self.isClient)
            local x,y,z = getWorldTranslation(bale.id)
            local rx,ry,rz = getWorldRotation(bale.id)
            baleObject:load(bale.filename, x,y,z,rx,ry,rz, bale.fillLevel)
            baleObject:register()
            delete(bale.id)
        end

        if (not self.hasBaleWrapper or self.moveBaleToWrapper == nil) and baleObject.nodeId ~= nil then
            -- release bale if there's no bale wrapper
            local x,y,z = getWorldTranslation(baleObject.nodeId)
            local vx,vy,vz = getVelocityAtWorldPos(self.cottonBaler.baleAnimRootComponent, x,y,z)
            setLinearVelocity(baleObject.nodeId, vx,vy,vz)
        elseif self.moveBaleToWrapper ~= nil then
            -- move bale to wrapper
            self:moveBaleToWrapper(baleObject)
        end
    end

    Utils.releaseSharedI3DFile(bale.filename, nil, true)
    table.remove(self.cottonBaler.bales, baleIndex)

    g_currentMission.missionStats:updateStats("baleCount", 1)
end

--function cottonBaler:getIsTurnedOnAllowed(superFunc, isTurnedOn)
--    if isTurnedOn and self.cottonBaler.isBaleUnloading then
--        return false
--    end

--    if superFunc ~= nil then
--        return superFunc(self, isTurnedOn)
--    end

--    return true
--end

function cottonBaler:loadWorkAreaFromXML(superFunc, workArea, xmlFile, key)
    -- fcelsa probabilmente non servirà...
    local retValue = true
    if superFunc ~= nil then
        retValue = superFunc(self, workArea, xmlFile, key)
    end

    if workArea.type == WorkArea.AREATYPE_DEFAULT then
        workArea.type = WorkArea.AREATYPE_BALER
    end

    return retValue
end

function cottonBaler:loadSpeedRotatingPartFromXML(superFunc, speedRotatingPart, xmlFile, key)
    if superFunc ~= nil then
        if not superFunc(self, speedRotatingPart, xmlFile, key) then
            return false
        end
    end

    speedRotatingPart.rotateOnlyIfFillLevelIncreased = Utils.getNoNil(getXMLBool(xmlFile, key .. "#rotateOnlyIfFillLevelIncreased"), false)

    return true
end

function cottonBaler:getIsSpeedRotatingPartActive(superFunc, speedRotatingPart)
    if speedRotatingPart.rotateOnlyIfFillLevelIncreased ~= nil then
        if speedRotatingPart.rotateOnlyIfFillLevelIncreased and not self.cottonBaler.lastAreaBiggerZero then
            return false
        end
    end

    if superFunc ~= nil then
        return superFunc(self, speedRotatingPart)
    end
    return true
end

-- fcelsa:  modified processBalerAreas, due to workarea not used here, so workarea as replaced with fillLevel of combine to be processed.
function cottonBaler:processBalerAreas(strawDropped, fillTypes)

    local totalLiters = 0
    local usedFillType = FillUtil.FILLTYPE_UNKNOWN

    if usedFillType == FillUtil.FILLTYPE_UNKNOWN then
        --for _, fillType in pairs(fillTypes) do
            local liters = -strawDropped;
            if liters > 0 then
                usedFillType = fillType
                totalLiters = totalLiters + liters
                --break
            end
        --end
        
    else
        totalLiters = totalLiters - strawDropped
    end

    return totalLiters, usedFillType
end
